# -*- coding: utf-8 -*-
"""Plone support for MLS Listings."""

from plone import api


PLONE_4 = '4' <= api.env.plone_version() < '5'
PLONE_5 = '5' <= api.env.plone_version() < '6'

PRODUCT_NAME = 'plone.mls.listing'


class AnnotationStorage(dict):
    """Custom annotation dict for MLS configurations."""

    context = None



# -*- coding: utf-8 -*-
"""MLS Listing collection."""

from plone.app.layout.viewlets.common import ViewletBase
from plone.directives import form
from plone.memoize.view import memoize
from plone.mls.core.navigation import ListingBatch
from plone.mls.listing import AnnotationStorage
from plone.mls.listing import PLONE_4
from plone.mls.listing import PLONE_5
from plone.mls.listing.api import prepare_search_params
from plone.mls.listing.api import search
from plone.mls.listing.browser.interfaces import IBaseListingItems
from plone.mls.listing.browser.interfaces import IListingDetails
from plone.mls.listing.i18n import _
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from z3c.form import button
from z3c.form import field
from z3c.form.browser import checkbox
from zope import schema
from zope.annotation.interfaces import IAnnotations
from zope.component import queryMultiAdapter
from zope.interface import alsoProvides
from zope.interface import Interface
from zope.interface import noLongerProvides
from zope.traversing.browser.absoluteurl import absoluteURL


CONFIGURATION_KEY = 'plone.mls.listing.listingcollection'


class IPossibleListingCollection(Interface):
    """Marker interface for possible ListingCollection viewlet."""


class IListingCollection(IBaseListingItems):
    """Marker interface for ListingCollection viewlet."""


class ListingCollectionViewlet(ViewletBase):
    """Dynamic collection of MLS listings."""

    _listings = None
    _batching = None

    if PLONE_5:
        index = ViewPageTemplateFile('templates/p5_listing_results.pt')
    elif PLONE_4:
        index = ViewPageTemplateFile('templates/recent_listings_viewlet.pt')

    @property
    def available(self):
        return IListingCollection.providedBy(self.context) and \
            not IListingDetails.providedBy(self.view)

    @property
    def config(self):
        """Get view configuration data from annotations."""
        annotations = IAnnotations(self.context)
        return annotations.get(CONFIGURATION_KEY, {})

    def update(self):
        """Prepare view related data."""
        super(ListingCollectionViewlet, self).update()
        self.context_state = queryMultiAdapter((self.context, self.request),
                                               name='plone_context_state')

        self.limit = self.config.get('limit', 25)
        self._get_listings()

    def _get_listings(self):
        """Query the recent listings from the MLS."""
        params = {
            'limit': self.limit,
            'offset': self.request.get('b_start', 0),
            'lang': self.portal_state.language(),
        }
        params.update(self.config)
        params = prepare_search_params(params)
        results, batching = search(
            params=params,
            context=self.context,
            config=self.config,
        )
        self._listings = results
        self._batching = batching

    @property
    @memoize
    def listings(self):
        """Return listing results."""
        return self._listings

    @memoize
    def view_url(self):
        """Generate view url."""
        if not self.context_state.is_view_template():
            url = self.context_state.current_base_url()
        else:
            url = absoluteURL(self.context, self.request)
        if not url.endswith('/'):
            url = url + '/'
        return url

    @property
    def batching(self):
        return ListingBatch(self.listings, self.limit,
                            self.request.get('b_start', 0), orphan=1,
                            batch_data=self._batching)


class IListingCollectionConfiguration(Interface):
    """Listing Collection Configuration Form."""

    sort_on = schema.Choice(
        default='last_activated_date',
        required=True,
        title=_(u'Sort by'),
        source='plone.mls.listing.SortIndices',
    )

    reverse = schema.Bool(
        default=True,
        required=False,
        title=(u'Reverse'),
    )

    overriding_agency_id = schema.TextLine(
        description=_(
            u'Specify an agency ID (or a comma-separated list of multiple '
            u'agency IDs) that will override the Base/Local MLS Settings. If '
            u'left blank, the agency ID from the Base/Local MLS settings will '
            u'be used.',
        ),
        required=False,
        title=_(u'Overriding Agency IDs'),
    )

    agency_listings = schema.Bool(
        description=_(
            u'If activated, only listings of the configured agency are shown.',
        ),
        required=False,
        title=_(u'Agency Listings'),
    )

    agency_priority = schema.Bool(
        description=_(
            u'If selected, the results will first display the listings from '
            u'this agency and then display the rest of the applicable '
            u'listings. This option will supersede the \'Agency Listings\' '
            u'option if selected.',
        ),
        required=False,
        title=_(u'Agency Priority Ordering'),
    )

    show_unverified = schema.Bool(
        default=False,
        required=False,
        title=_(u'Show Unverified Listings'),
    )

    show_unverified_only = schema.Bool(
        description=_(
            u'"Show Unverified Listings" must be activated to take effect.',
        ),
        default=False,
        required=False,
        title=_(u'Only Show Unverified Listings'),
    )

    listing_type = schema.Tuple(
        default=('cl', 'cs', 'll', 'rl', 'rs'),
        required=False,
        title=_(u'Listing Type'),
        value_type=schema.Choice(
            source='plone.mls.listing.ListingTypes',
        ),
    )

    workflow_state = schema.Tuple(
        default=('Active', 'PendingSale', 'Sold'),
        required=False,
        title=_(u'Workflow Status'),
        value_type=schema.Choice(
            source='plone.mls.listing.WorkflowStates',
        ),
    )

    location_state = schema.Tuple(
        required=False,
        title=_(u'State'),
        value_type=schema.Choice(
            source='plone.mls.listing.LocationStates',
        ),
    )

    location_county = schema.Tuple(
        required=False,
        title=_(u'County'),
        value_type=schema.Choice(
            source='plone.mls.listing.LocationCounties',
        ),
    )

    location_district = schema.Tuple(
        required=False,
        title=_(u'District'),
        value_type=schema.Choice(
            source='plone.mls.listing.LocationDistricts',
        ),
    )

    location_city = schema.TextLine(
        required=False,
        title=_(u'City/Town'),
    )

    price_min = schema.Int(
        required=False,
        title=_(u'Price (Min)'),
    )

    price_max = schema.Int(
        required=False,
        title=_(u'Price (Max)'),
    )

    lot_size_min = schema.Int(
        required=False,
        title=_(u'Lot Size in m² (Min)'),
    )

    lot_size_max = schema.Int(
        required=False,
        title=_(u'Lot Size in m² (Max)'),
    )

    floor_area_min = schema.Int(
        required=False,
        title=_(u'Interior Area in m² (Min)'),
        description=_(
            u'Reminder: All land listings will be excluded once any value is '
            u'set.',
        ),
    )

    floor_area_max = schema.Int(
        required=False,
        title=_(u'Interior Area in m² (Max)'),
    )

    beds_min = schema.Int(
        required=False,
        title=_(u'Bedrooms (Min)'),
        description=_(
            u'Reminder: All land listings will be excluded once any value is '
            u'set as well as any commercial listings with no bedrooms.',
        ),
    )

    beds_max = schema.Int(
        required=False,
        title=_(u'Bedrooms (Max)'),
    )

    baths_min = schema.Int(
        required=False,
        title=_(u'Bathrooms (Min)'),
        description=_(
            u'Reminder: All land listings will be excluded once any value is '
            u'set as well as any commercial listings with no bathrooms.',
        ),
    )

    baths_max = schema.Int(
        required=False,
        title=_(u'Bathrooms (Max)'),
    )

    location_type = schema.Tuple(
        required=False,
        title=_(u'Location Type'),
        value_type=schema.Choice(
            source='plone.mls.listing.LocationTypes',
        ),
    )

    geographic_type = schema.Tuple(
        required=False,
        title=_(u'Geographic Type'),
        value_type=schema.Choice(
            source='plone.mls.listing.GeographicTypes',
        ),
    )

    view_type = schema.Tuple(
        required=False,
        title=_(u'View Type'),
        value_type=schema.Choice(
            source='plone.mls.listing.ViewTypes',
        ),
    )

    object_type = schema.Tuple(
        required=False,
        title=_(u'Object Type'),
        value_type=schema.Choice(
            source='plone.mls.listing.ObjectTypes',
        ),
    )

    ownership_type = schema.Tuple(
        required=False,
        title=_(u'Ownership Type'),
        value_type=schema.Choice(
            source='plone.mls.listing.OwnershipTypes',
        ),
    )

    limit = schema.Int(
        default=25,
        required=False,
        title=_(u'Items per Page'),
    )

    zoomlevel = schema.Int(
        default=7,
        required=False,
        title=_(u'Zoomlevel for google-maps '),
    )


class ListingCollectionConfiguration(form.Form):
    """Listing Collection Configuration Form."""

    fields = field.Fields(IListingCollectionConfiguration)
    fields['geographic_type'].widgetFactory = checkbox.CheckBoxFieldWidget
    fields['listing_type'].widgetFactory = checkbox.CheckBoxFieldWidget
    fields['workflow_state'].widgetFactory = checkbox.CheckBoxFieldWidget
    fields['location_type'].widgetFactory = checkbox.CheckBoxFieldWidget
    fields['object_type'].widgetFactory = checkbox.CheckBoxFieldWidget
    fields['ownership_type'].widgetFactory = checkbox.CheckBoxFieldWidget
    fields['view_type'].widgetFactory = checkbox.CheckBoxFieldWidget

    label = _(u"'Listing Collection' Configuration")
    description = _(
        u"Adjust the behaviour for this 'Listing Collection' viewlet.",
    )

    def getContent(self):
        annotations = IAnnotations(self.context)
        content = annotations.get(
            CONFIGURATION_KEY,
            annotations.setdefault(CONFIGURATION_KEY, {}),
        )
        content = AnnotationStorage(content)
        content.context = self.context
        return content

    @button.buttonAndHandler(_(u'Save'))
    def handle_save(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        annotations = IAnnotations(self.context)
        annotations[CONFIGURATION_KEY] = data
        self.request.response.redirect(absoluteURL(self.context, self.request))

    @button.buttonAndHandler(_(u'Cancel'))
    def handle_cancel(self, action):
        self.request.response.redirect(absoluteURL(self.context, self.request))


class ListingCollectionStatus(object):
    """Return activation/deactivation status of ListingCollection viewlet."""

    def __init__(self, context, request):
        self.context = context
        self.request = request

    @property
    def can_activate(self):
        return IPossibleListingCollection.providedBy(self.context) and \
            not IListingCollection.providedBy(self.context)

    @property
    def active(self):
        return IListingCollection.providedBy(self.context)


class ListingCollectionToggle(object):
    """Toggle ListingCollection viewlet for the current context."""

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        msg_type = 'info'

        if IListingCollection.providedBy(self.context):
            # Deactivate ListingCollection viewlet.
            noLongerProvides(self.context, IListingCollection)
            self.context.reindexObject(idxs=['object_provides'])
            msg = _(u"'Listing Collection' viewlet deactivated.")
        elif IPossibleListingCollection.providedBy(self.context):
            alsoProvides(self.context, IListingCollection)
            self.context.reindexObject(idxs=['object_provides'])
            msg = _(u"'Listing Collection' viewlet activated.")
        else:
            msg = _(
                u'The \'Listing Collection\' viewlet does\'t work with this '
                u'content type. Add \'IPossibleListingCollection\' to the '
                u'provided interfaces to enable this feature.',
            )
            msg_type = 'error'

        self.context.plone_utils.addPortalMessage(msg, msg_type)
        self.request.response.redirect(self.context.absolute_url())



# -*- coding: utf-8 -*-
"""Test Layer for plone.mls.listing."""

# zope imports
from plone.app.testing import applyProfile
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer


class PloneMLSListing(PloneSandboxLayer):
    """Custom Test Layer for plone.mls.listing."""
    defaultBases = (PLONE_FIXTURE, )

    def setUpZope(self, app, configurationContext):
        """Set up Zope for testing."""
        # Load ZCML
        import plone.mls.listing
        self.loadZCML(package=plone.mls.listing)

    def setUpPloneSite(self, portal):
        """Set up a Plone site for testing."""
        applyProfile(portal, 'plone.mls.listing:default')


PLONE_MLS_LISTING_FIXTURE = PloneMLSListing()
PLONE_MLS_LISTING_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONE_MLS_LISTING_FIXTURE, ),
    name='PloneMLSListing:Integration',
)



# -*- coding: utf-8 -*-
"""View adapter."""

from plone.mls.listing import browser
from plone.mls.listing.i18n import _
from z3c.form.widget import StaticWidgetAttribute


LABEL_ALL = _(u'All')


PleaseSelectLCC = StaticWidgetAttribute(
    LABEL_ALL,
    view=browser.listing_collection.ListingCollectionConfiguration,
)
PleaseSelectLSF = StaticWidgetAttribute(
    LABEL_ALL,
    view=browser.listing_search.ListingSearchForm,
)
PleaseSelectRLC = StaticWidgetAttribute(
    LABEL_ALL,
    view=browser.recent_listings.RecentListingsConfiguration,
)


PleaseSelectState = StaticWidgetAttribute(
    LABEL_ALL,
    field=browser.listing_search.IListingSearchForm['location_state'],
)
PleaseSelectCounty = StaticWidgetAttribute(
    LABEL_ALL,
    field=browser.listing_search.IListingSearchForm['location_county'],
)
PleaseSelectDistrict = StaticWidgetAttribute(
    LABEL_ALL,
    field=browser.listing_search.IListingSearchForm['location_district'],
)



# -*- coding: utf-8 -*-
"""Setup handlers for plone.mls.listing."""

from plone.mls.listing import PLONE_4
from Products.CMFPlone.interfaces import INonInstallable
from zope.interface import implementer


@implementer(INonInstallable)
class HiddenProfiles(object):

    def getNonInstallableProfiles(self):
        """Hide uninstall profile from site-creation and quickinstaller."""
        items = [
            'plone.mls.listing:uninstall',
        ]
        if not PLONE_4:
            items.append('plone.mls.listing:default')

        return items



# -*- coding: utf-8 -*-
"""Test Setup of plone.mls.listing."""

from plone.mls.listing import PLONE_4
from plone.mls.listing.testing import PLONE_MLS_LISTING_INTEGRATION_TESTING


try:
    import unittest2 as unittest
except ImportError:
    import unittest


class TestSetup(unittest.TestCase):
    """Setup Test Case for plone.mls.listing."""
    layer = PLONE_MLS_LISTING_INTEGRATION_TESTING

    def test_collective_prettyphoto_installed(self):
        """Validate that collective.prettyphoto is installed."""
        if not PLONE_4:
            return

        portal = self.layer['portal']
        qi = portal.portal_quickinstaller
        self.assertTrue(qi.isProductInstalled('collective.prettyphoto'))

    def test_plone_app_dexterity_installed(self):
        """Validate that plone.app.dexterity is installed."""
        portal = self.layer['portal']
        qi = portal.portal_quickinstaller
        if qi.isProductAvailable('plone.app.dexterity'):
            self.assertTrue(qi.isProductInstalled('plone.app.dexterity'))
        else:
            self.assertTrue(
                'plone.app.dexterity' in qi.listInstallableProfiles(),
            )

    def test_plone_mls_core_installed(self):
        """Validate that plone.mls.core is installed."""
        portal = self.layer['portal']
        qi = portal.portal_quickinstaller
        self.assertTrue(qi.isProductInstalled('plone.mls.core'))



# -*- coding: utf-8 -*-
"""MLS Listing Search."""

from Acquisition import aq_inner
from plone import api
from plone.app.layout.viewlets.common import ViewletBase
from plone.autoform import directives
from plone.directives import form
from plone.memoize.view import memoize
from plone.mls.core.navigation import ListingBatch
from plone.mls.listing import AnnotationStorage
from plone.mls.listing import PLONE_4
from plone.mls.listing import PLONE_5
from plone.mls.listing.api import prepare_search_params
from plone.mls.listing.api import search
from plone.mls.listing.browser.interfaces import IBaseListingItems
from plone.mls.listing.browser.interfaces import IListingDetails
from plone.mls.listing.browser.valuerange.widget import ValueRangeFieldWidget
from plone.mls.listing.i18n import _
from plone.portlets.interfaces import IPortletManager
from plone.portlets.interfaces import IPortletRetriever
from plone.supermodel import model
from plone.z3cform import z2
from Products.CMFPlone import PloneMessageFactory as PMF  # noqa
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from z3c.form import button
from z3c.form.browser import checkbox
from z3c.form.browser import radio
from z3c.form.interfaces import IFormLayer
from zope import schema
from zope.annotation.interfaces import IAnnotations
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.interface import alsoProvides
from zope.interface import Interface
from zope.interface import noLongerProvides
from zope.traversing.browser.absoluteurl import absoluteURL


# starting from 0.6.0 version plone.z3cform has IWrappedForm interface
try:
    from plone.z3cform.interfaces import IWrappedForm
    HAS_WRAPPED_FORM = True
except ImportError:
    HAS_WRAPPED_FORM = False


CONFIGURATION_KEY = 'plone.mls.listing.listingsearch'

FIELD_ORDER = {
    'row_listing_type': [
        'q',
        'listing_type',
    ],
    'row_location': [
        'location_state',
        'location_county',
        'location_district',
    ],
    'row_price': [
        'location_city',
        'price_min',
        'price_max',
    ],
    'row_beds_baths': [
        'beds',
        'baths',
    ],
    'row_sizes': [
        'lot_size',
        'interior_area',
    ],
    'row_other': [
        'air_condition',
        'pool',
        'jacuzzi',
    ],
    'row_tabbed': [
        'location_type',
        'geographic_type',
        'view_type',
        'object_type',
        'ownership_type',
    ],
}

EXCLUDED_SEARCH_FIELDS = [
    'auto_search',
    'hide_form',
    'zoomlevel',
]


def encode_dict(in_dict):
    """Encode dict values to utf-8."""
    out_dict = {}
    for k, v in in_dict.iteritems():
        if isinstance(v, unicode):
            v = v.encode('utf8')
        elif isinstance(v, str):
            # Must be encoded in UTF-8
            v.decode('utf8')
        out_dict[k] = v
    return out_dict


class IPossibleListingSearch(Interface):
    """Marker interface for possible ListingSearch viewlet."""


class IListingSearch(IBaseListingItems):
    """Marker interface for ListingSearch viewlet."""


class IListingSearchForm(model.Schema):
    """Listing search form schema definition."""

    if PLONE_5:
        from plone.app.z3cform.widget import SelectFieldWidget
        form.widget(
            'location_state',
            SelectFieldWidget,
            pattern_options={'placeholder': _(u'Select a State')},
        )
        form.widget(
            'location_county',
            SelectFieldWidget,
            pattern_options={'placeholder': _(u'Select a County')},
        )
        form.widget(
            'location_district',
            SelectFieldWidget,
            pattern_options={'placeholder': _(u'Select a District')},
        )

    q = schema.TextLine(
        required=False,
        title=_(u'Freetext search (Location, Keywords, Listing ID, ...)'),
    )

    form.widget(listing_type=checkbox.CheckBoxFieldWidget)
    listing_type = schema.Tuple(
        required=False,
        title=_(u'Listing Type'),
        value_type=schema.Choice(
            source='plone.mls.listing.ListingTypesSearch',
        ),
    )

    location_state = schema.Choice(
        required=False,
        title=_(u'State'),
        source='plone.mls.listing.LocationStates',
    )

    location_county = schema.Choice(
        required=False,
        title=_(u'County'),
        source='plone.mls.listing.LocationCounties',
    )

    location_district = schema.Choice(
        required=False,
        title=_(u'District'),
        source='plone.mls.listing.LocationDistricts',
    )

    location_city = schema.TextLine(
        required=False,
        title=_(u'City/Town'),
    )

    price_min = schema.Int(
        required=False,
        title=_(u'Price (Min)'),
    )

    price_max = schema.Int(
        required=False,
        title=_(u'Price (Max)'),
    )

    form.widget(location_type=checkbox.CheckBoxFieldWidget)
    location_type = schema.Tuple(
        required=False,
        title=_(u'Location Type'),
        value_type=schema.Choice(
            source='plone.mls.listing.LocationTypes',
        ),
    )

    form.widget(geographic_type=checkbox.CheckBoxFieldWidget)
    geographic_type = schema.Tuple(
        required=False,
        title=_(u'Geographic Type'),
        value_type=schema.Choice(
            source='plone.mls.listing.GeographicTypes',
        ),
    )

    form.widget(view_type=checkbox.CheckBoxFieldWidget)
    view_type = schema.Tuple(
        required=False,
        title=_(u'View Type'),
        value_type=schema.Choice(
            source='plone.mls.listing.ViewTypes',
        ),
    )

    form.widget(object_type=checkbox.CheckBoxFieldWidget)
    object_type = schema.Tuple(
        required=False,
        title=_(u'Object Type'),
        value_type=schema.Choice(
            source='plone.mls.listing.ObjectTypes',
        ),
    )

    form.widget(ownership_type=checkbox.CheckBoxFieldWidget)
    ownership_type = schema.Tuple(
        required=False,
        title=_(u'Ownership Type'),
        value_type=schema.Choice(
            source='plone.mls.listing.OwnershipTypes',
        ),
    )

    form.widget(beds=ValueRangeFieldWidget)
    beds = schema.Tuple(
        default=('--MINVALUE--', '--MAXVALUE--'),
        required=False,
        title=_(u'Bedrooms'),
        value_type=schema.Choice(
            source='plone.mls.listing.Rooms',
        ),
    )

    form.widget(baths=ValueRangeFieldWidget)
    baths = schema.Tuple(
        default=('--MINVALUE--', '--MAXVALUE--'),
        required=False,
        title=_(u'Bathrooms'),
        value_type=schema.Choice(
            source='plone.mls.listing.Rooms',
        ),
    )

    form.widget(air_condition=radio.RadioFieldWidget)
    air_condition = schema.Choice(
        default='--NOVALUE--',
        required=False,
        source='plone.mls.listing.YesNoAll',
        title=_(u'Air Condition'),
    )

    form.widget(pool=radio.RadioFieldWidget)
    pool = schema.Choice(
        default='--NOVALUE--',
        required=False,
        source='plone.mls.listing.YesNoAll',
        title=_(u'Pool'),
    )

    form.widget(jacuzzi=radio.RadioFieldWidget)
    jacuzzi = schema.Choice(
        default='--NOVALUE--',
        required=False,
        source='plone.mls.listing.YesNoAll',
        title=_(u'Jacuzzi'),
    )

    form.widget(lot_size=ValueRangeFieldWidget)
    lot_size = schema.Tuple(
        default=('--MINVALUE--', '--MAXVALUE--'),
        required=False,
        title=_(u'Lot Size'),
        value_type=schema.Choice(
            source='plone.mls.listing.LotSizes',
        ),
    )

    form.widget(interior_area=ValueRangeFieldWidget)
    interior_area = schema.Tuple(
        default=('--MINVALUE--', '--MAXVALUE--'),
        required=False,
        title=_(u'Interior Area'),
        value_type=schema.Choice(
            source='plone.mls.listing.InteriorAreaSizes',
        ),
    )


class ListingSearchForm(form.SchemaForm):
    """Listing Search Form."""

    schema = IListingSearchForm

    if PLONE_5:
        template = ViewPageTemplateFile('templates/p5_search_form.pt')
    elif PLONE_4:
        template = ViewPageTemplateFile('templates/search_form.pt')
    ignoreContext = True
    method = 'get'

    def __init__(self, context, request):
        """Customized form constructor."""
        super(ListingSearchForm, self).__init__(context, request)
        self.prefix = 'form.{0}'.format(self.context.id)
        self.omitted = []

    def update(self):
        super(ListingSearchForm, self).update()
        if self.config.get('location_city', None):
            self.omitted.extend([
                'location_state',
                'location_county',
                'location_district',
                'location_city',
            ])

        if self.config.get('location_district', None):
            self.omitted.extend([
                'location_state',
                'location_county',
                'location_district',
            ])
        elif self.config.get('location_county', None):
            self.omitted.extend([
                'location_state',
                'location_county',
                'location_district',
            ])
        elif self.config.get('location_state', None):
            self.omitted.extend([
                'location_state',
                'location_county',
                'location_district',
            ])

    @button.buttonAndHandler(PMF(u'label_search', default=u'Search'),
                             name='search')
    def handle_search(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

    def _widgets(self, row):
        """Return a list of widgets that should be shown for a given row."""
        widget_data = dict(self.widgets.items())
        available_fields = FIELD_ORDER.get(row, [])
        return [
            widget_data.get(key, None) for key in available_fields if
            key not in self.omitted
        ]

    def widgets_listing_type(self):
        """Return the widgets for the row ``row_listing_type``."""
        return self._widgets('row_listing_type')

    def widgets_location(self):
        """Return the widgets for the row ``row_location``."""
        return self._widgets('row_location')

    def widgets_price(self):
        """Return the widgets for the row ``row_price``."""
        return self._widgets('row_price')

    def widgets_beds_baths(self):
        """Return the widgets for the row ``row_beds_baths``."""
        return self._widgets('row_beds_baths')

    def widgets_sizes(self):
        """Return the widgets for the row ``row_sizes``."""
        return self._widgets('row_sizes')

    def widgets_other(self):
        """Return the widgets for the row ``row_other``."""
        return self._widgets('row_other')

    def widgets_outstanding(self):
        """Return all other widgets that have not been shown until now."""
        defined_fields = FIELD_ORDER.values()
        shown_fields = [
            shown_field for field_lists in defined_fields for
            shown_field in field_lists
        ]
        return [
            widget for field_name, widget in self.widgets.items() if
            field_name not in shown_fields
        ]

    def widgets_tabbed(self):
        """Return the widgets for the row ``row_tabbed``."""
        return self._widgets('row_tabbed')

    @property
    def config(self):
        """Get view configuration data from annotations."""
        annotations = IAnnotations(self.context)
        return annotations.get(CONFIGURATION_KEY, {})


class ListingSearchViewlet(ViewletBase):
    """Search for listings in the MLS."""

    _listings = None
    _batching = None

    if PLONE_5:
        index = ViewPageTemplateFile('templates/p5_listing_search_viewlet.pt')
    elif PLONE_4:
        index = ViewPageTemplateFile('templates/listing_search_viewlet.pt')

    @property
    def available(self):
        return IListingSearch.providedBy(self.context) and \
            not IListingDetails.providedBy(self.view)

    @property
    def config(self):
        """Get view configuration data from annotations."""
        annotations = IAnnotations(self.context)
        return annotations.get(CONFIGURATION_KEY, {})

    @property
    def hide_form(self):
        if self.config.get('hide_form', True) is False:
            return False

        if self.form and not self.search_performed:
            return False

        from plone.mls.listing.portlets.quick_search import IQuickSearchPortlet
        portlets = []
        for column in ['plone.leftcolumn', 'plone.rightcolumn']:
            manager = manager = getUtility(IPortletManager, name=column)
            retriever = queryMultiAdapter((self.context, manager),
                                          IPortletRetriever)
            portlets.extend(retriever.getPortlets())
        return len([
            portlet for portlet in portlets if
            IQuickSearchPortlet.providedBy(portlet['assignment']) and
            self.is_filter_portlet(portlet['assignment'])
        ]) > 0

    def is_filter_portlet(self, portlet_assignment):
        search_path = portlet_assignment.target_search

        if search_path is None:
            return False

        if search_path.startswith('/'):
            obj = api.content.get(path=search_path)
        else:
            obj = api.content.get(UID=search_path)
        if obj:
            return obj == self.context
        return False

    def update(self):
        """Prepare view related data."""
        super(ListingSearchViewlet, self).update()
        self.context_state = queryMultiAdapter((self.context, self.request),
                                               name='plone_context_state')

        self.limit = self.config.get('limit', 25)

        z2.switch_on(self, request_layer=IFormLayer)
        self.form = ListingSearchForm(aq_inner(self.context), self.request)
        if HAS_WRAPPED_FORM:
            alsoProvides(self.form, IWrappedForm)
        self.form.update()
        button = '{0}.buttons.search'.format(self.form.prefix)

        self.search_performed = (
            button in self.request.keys() or
            self.config.get('auto_search', False)
        )
        if self.available and self.search_performed:
            data, errors = self.form.extractData()
            if not errors:
                self._get_listings(prepare_search_params(data))

            self.request.form = encode_dict(self.request.form)

    def _get_listings(self, params):
        """Query the recent listings from the MLS."""
        search_params = {
            'limit': self.limit,
            'offset': self.request.get('b_start', 0),
            'lang': self.portal_state.language(),
        }

        search_params.update(self.config)
        search_params.update(params)
        search_params = prepare_search_params(
            search_params,
            omit=EXCLUDED_SEARCH_FIELDS,
        )
        results, batching = search(
            params=search_params,
            context=self.context,
            config=self.config,
        )
        self._listings = results
        self._batching = batching

    @property
    @memoize
    def listings(self):
        """Return listing results."""
        return self._listings

    @memoize
    def view_url(self):
        """Generate view url."""
        if not self.context_state.is_view_template():
            url = self.context_state.current_base_url()
        else:
            url = absoluteURL(self.context, self.request)
        if not url.endswith('/'):
            url = url + '/'
        return url

    @property
    def batching(self):
        return ListingBatch(self.listings, self.limit,
                            self.request.get('b_start', 0), orphan=1,
                            batch_data=self._batching)


class IListingSearchConfiguration(model.Schema):
    """Listing Search Configuration Form."""

    agency_listings = schema.Bool(
        description=_(
            u'If activated, only listings of the configured agency are shown.',
        ),
        required=False,
        title=_(u'Agency Listings'),
    )

    agency_priority = schema.Bool(
        description=_(
            u'If selected, the results will first display the listings from '
            u'this agency and then display the rest of the applicable '
            u'listings. This option will supersede the \'Agency Listings\' '
            u'option if selected.',
        ),
        required=False,
        title=_(u'Agency Priority Ordering'),
    )

    show_unverified = schema.Bool(
        default=False,
        required=False,
        title=_(u'Show Unverified Listings'),
    )

    show_unverified_only = schema.Bool(
        description=_(
            u'"Show Unverified Listings" must be activated to take effect.',
        ),
        default=False,
        required=False,
        title=_(u'Only Show Unverified Listings'),
    )

    limit = schema.Int(
        default=25,
        required=False,
        title=_(u'Items per Page'),
    )

    zoomlevel = schema.Int(
        default=7,
        required=False,
        title=_(u'Zoomlevel for google-maps '),
    )

    hide_form = schema.Bool(
        default=True,
        required=False,
        title=_(
            u'Hide the search form when Quick Search Portlet is available?',
        ),
    )

    auto_search = schema.Bool(
        default=False,
        required=False,
        title=_(
            u'Start the search immediately, without showing the search '
            u'form first?',
        ),
    )

    directives.widget(listing_type=checkbox.CheckBoxFieldWidget)
    listing_type = schema.Tuple(
        description=_(
            u'Select the available listing types for this search. If nothing '
            u'is selected, all available listing types will be shown.',
        ),
        required=False,
        title=_(u'Listing Type'),
        value_type=schema.Choice(
            source='plone.mls.listing.ListingTypes',
        ),
    )

    directives.widget(workflow_state=checkbox.CheckBoxFieldWidget)
    workflow_state = schema.Tuple(
        default=('Active', 'PendingSale', 'Sold'),
        required=False,
        title=_(u'Workflow Status'),
        value_type=schema.Choice(
            source='plone.mls.listing.WorkflowStates',
        ),
    )

    location_state = schema.Tuple(
        required=False,
        title=_(u'State'),
        value_type=schema.Choice(
            source='plone.mls.listing.LocationStates',
        ),
    )

    location_county = schema.Tuple(
        required=False,
        title=_(u'County'),
        value_type=schema.Choice(
            source='plone.mls.listing.LocationCounties',
        ),
    )

    location_district = schema.Tuple(
        required=False,
        title=_(u'District'),
        value_type=schema.Choice(
            source='plone.mls.listing.LocationDistricts',
        ),
    )

    location_city = schema.TextLine(
        required=False,
        title=_(u'City/Town'),
    )


class ListingSearchConfiguration(form.SchemaForm):
    """Listing Search Configuration Form."""

    prefix = 'form.config'
    schema = IListingSearchConfiguration
    label = _(u"'Listing Search' Configuration")
    description = _(u"Adjust the behaviour for this 'Listing Search' viewlet.")

    def getContent(self):
        annotations = IAnnotations(self.context)
        content = annotations.get(
            CONFIGURATION_KEY,
            annotations.setdefault(CONFIGURATION_KEY, {}),
        )
        content = AnnotationStorage(content)
        content.context = self.context
        return content

    @button.buttonAndHandler(_(u'Save'))
    def handle_save(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        annotations = IAnnotations(self.context)
        annotations[CONFIGURATION_KEY] = data
        self.request.response.redirect(absoluteURL(self.context, self.request))
        return u''

    @button.buttonAndHandler(_(u'Cancel'))
    def handle_cancel(self, action):
        self.request.response.redirect(absoluteURL(self.context, self.request))
        return u''


class ListingSearchStatus(object):
    """Return activation/deactivation status of ListingSearch viewlet."""

    def __init__(self, context, request):
        self.context = context
        self.request = request

    @property
    def can_activate(self):
        return IPossibleListingSearch.providedBy(self.context) and \
            not IListingSearch.providedBy(self.context)

    @property
    def active(self):
        return IListingSearch.providedBy(self.context)


class ListingSearchToggle(object):
    """Toggle ListingSearch viewlet for the current context."""

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        msg_type = 'info'

        if IListingSearch.providedBy(self.context):
            # Deactivate ListingSearch viewlet.
            noLongerProvides(self.context, IListingSearch)
            self.context.reindexObject(idxs=['object_provides'])
            msg = _(u"'Listing Search' viewlet deactivated.")
        elif IPossibleListingSearch.providedBy(self.context):
            alsoProvides(self.context, IListingSearch)
            self.context.reindexObject(idxs=['object_provides'])
            msg = _(u"'Listing Search' viewlet activated.")
        else:
            msg = _(
                u'The \'Listing Search\' viewlet does\'t work with this '
                u'content type. Add \'IPossibleListingSearch\' to the '
                u'provided interfaces to enable this feature.',
            )
            msg_type = 'error'

        self.context.plone_utils.addPortalMessage(msg, msg_type)
        self.request.response.redirect(self.context.absolute_url())



# -*- coding: utf-8 -*-
"""Agent Contact Portlet."""

from Acquisition import aq_inner
from email import message_from_string
from email.utils import formataddr
from email.utils import getaddresses
from plone import api
from plone.app.portlets.portlets import base
from plone.directives import form
from plone.formwidget.captcha.validator import CaptchaValidator
from plone.formwidget.captcha.widget import CaptchaFieldWidget
from plone.mls.listing import PLONE_4
from plone.mls.listing import PLONE_5
from plone.mls.listing import PRODUCT_NAME
from plone.mls.listing.browser.interfaces import IListingDetails
from plone.mls.listing.browser.tcwidget.widget import TCFieldWidget
from plone.mls.listing.i18n import _
from plone.portlets.interfaces import IPortletDataProvider
from plone.z3cform import z2
from Products.CMFPlone import PloneMessageFactory as PMF
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from z3c.form import button
from z3c.form import field
from z3c.form import validator
from z3c.form.interfaces import HIDDEN_MODE
from z3c.form.interfaces import IFormLayer
from zope import schema
from zope.i18n import translate
from zope.interface import alsoProvides
from zope.interface import implementer
from zope.interface import Interface
from zope.interface import Invalid
from zope.schema.fieldproperty import FieldProperty

import copy
import logging
import re


# starting from 0.6.0 version plone.z3cform has IWrappedForm interface
try:
    from plone.z3cform.interfaces import IWrappedForm
    HAS_WRAPPED_FORM = True
except ImportError:
    HAS_WRAPPED_FORM = False

if PLONE_5:
    from plone.app.vocabularies.catalog import CatalogSource
elif PLONE_4:
    from plone.app.form.widgets.uberselectionwidget import UberSelectionWidget
    from plone.app.vocabularies.catalog import SearchableTextSourceBinder
    from zope import formlib


MSG_PORTLET_DESCRIPTION = _(
    u'This portlet shows a form to contact the corresponding agent for a '
    u'given listing via email.',
)

EMAIL_TEMPLATE = _(
    u'agent_contact_email',
    default=u'Enquiry from: {name} <{sender_from_address}>\n'
    u'Listing URL: {url}\n'
    u'\n'
    u'{form_data}\n'
    u'\n'
    u'Message:\n'
    u'{message}',
)

EMAIL_TEMPLATE_AGENT = _(
    u'agent_contact_email_agent',
    default=u'The responsible agent for this listing is '
    u'{agent} ({profile}).\n'
    u'\n'
    u'Please contact {agent} at {agent_email}.',
)


check_email = re.compile(
    r'[a-zA-Z0-9._%-]+@([a-zA-Z0-9-]+\.)*[a-zA-Z]{2,4}').match

check_for_url = re.compile(
    r'http[s]?://').search

logger = logging.getLogger(PRODUCT_NAME)


def validate_accept(value):
    if value is not True:
        return False
    return True


def validate_email(value):
    if value:
        if not check_email(value):
            raise Invalid(_(u'Invalid email address'))
    return True


def contains_nuts(value):
    """Check for traces of nuts, like urls or other spammer fun things"""
    if value:
        if check_for_url(value):
            raise Invalid(_(u'No Urls allowed'))
    return True


class IEmailForm(Interface):
    """Email Form schema."""

    subject = schema.TextLine(
        required=False,
        title=PMF(u'label_subject', default=u'Subject'),
    )

    name = schema.TextLine(
        description=PMF(
            u'help_sender_fullname',
            default=u'Please enter your full name',
        ),
        required=True,
        title=PMF(u'label_name', default=u'Name'),
    )

    sender_from_address = schema.TextLine(
        constraint=validate_email,
        description=PMF(
            u'help_sender_from_address',
            default=u'Please enter your e-mail address',
        ),
        required=True,
        title=PMF(u'label_sender_from_address', default=u'E-Mail'),
    )

    country = schema.TextLine(
        description=_(u'Please enter your country of residence.'),
        required=True,
        title=_(u'Country'),
    )

    zipcode = schema.TextLine(
        description=_(u'Please enter your ZIP code.'),
        required=False,
        title=_(u'ZIP'),
    )

    phone = schema.TextLine(
        description=_(
            u'Please enter a phone number. Some agents will not respond '
            u'without one.',
        ),
        missing_value=u'-',
        required=False,
        title=_(u'Phone Number'),
    )

    arrival_date = schema.TextLine(
        missing_value=u'-',
        required=False,
        title=_(u'Arrival Date'),
    )

    departure_date = schema.TextLine(
        missing_value=u'-',
        required=False,
        title=_(u'Departure Date'),
    )

    adults = schema.TextLine(
        missing_value=u'-',
        required=False,
        title=_(u'Adults'),
    )

    children = schema.TextLine(
        missing_value=u'-',
        required=False,
        title=_(u'Children'),
    )

    message = schema.Text(
        constraint=contains_nuts,
        description=PMF(
            u'help_message',
            default=u'Please enter the message you want to send.',
        ),
        max_length=1000,
        required=True,
        title=PMF(u'label_message', default=u'Message'),
    )

    captcha = schema.TextLine(
        required=True,
        title=_(u'Captcha'),
    )

    accept_tcs = schema.Bool(
        constraint=validate_accept,
        required=True,
        title=_(u'I accept the Terms & Conditions'),
    )


class EmailForm(form.Form):
    """Email Form."""
    fields = field.Fields(IEmailForm).omit(
        'country',
        'zipcode',
        'accept_tcs',
        'arrival_date',
        'departure_date',
        'adults',
        'children',
    )
    ignoreContext = True
    method = 'post'
    _email_sent = False

    def __init__(self, context, request, portlet_hash=None, info=None,
                 data=None):
        super(EmailForm, self).__init__(context, request)
        self.listing_info = info
        self.data = data
        if portlet_hash:
            self.prefix = portlet_hash + '.' + self.prefix
        self.check_for_spam = data.reject_links

    @property
    def already_sent(self):
        return self._email_sent

    @property
    def is_residential_lease(self):
        return self.listing_info.get('listing_id', '').lower().startswith('rl')

    def update(self):
        omitted = []
        if not self.is_residential_lease:
            omitted = [
                'arrival_date',
                'departure_date',
                'adults',
                'children',
            ]
        if not self.data.country_visible:
            omitted.append('country')
        if not self.data.zipcode_visible:
            omitted.append('zipcode')
        if not self.data.accept_tcs_visible:
            omitted.append('accept_tcs')
        if not self.data.captcha_visible:
            omitted.append('captcha')

        self.fields = field.Fields(IEmailForm).omit(*omitted)

        if 'accept_tcs' in self.fields:
            self.fields['accept_tcs'].widgetFactory = TCFieldWidget

        if 'captcha' in self.fields:
            self.fields['captcha'].widgetFactory = CaptchaFieldWidget
        super(EmailForm, self).update()

    def updateWidgets(self):
        super(EmailForm, self).updateWidgets()
        portal = api.portal.get()
        subject = u'{portal_title}: {title} ({lid})'.format(
            lid=self.listing_info['listing_id'],
            portal_title=portal.getProperty('title').decode('utf-8'),
            title=self.listing_info['listing_title'],
        )
        self.widgets['subject'].mode = HIDDEN_MODE
        self.widgets['subject'].value = subject
        if 'accept_tcs' in self.widgets:
            self.widgets['accept_tcs'].target = self.data.accept_tcs_target

        if not self.check_for_spam:
            schema_field = copy.copy(self.widgets['message'].field)
            schema_field.constraint = lambda x: True
            self.widgets['message'].field = schema_field

        if self.data.phone_required:
            phone_field = copy.copy(self.widgets['phone'].field)
            phone_field.required = True
            self.widgets['phone'].field = phone_field
            self.widgets['phone'].required = True

    @button.buttonAndHandler(PMF(u'label_send', default='Send'), name='send')
    def handle_send(self, action):
        """Send button for sending the email."""
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        if 'captcha' in data:
            # Verify the user input against the captcha
            captcha = CaptchaValidator(
                self.context, self.request, None, IEmailForm['captcha'], None,
            )
            if not self.already_sent and captcha.validate(data['captcha']):
                self.send_email(data)
                self._email_sent = True
        else:
            if not self.already_sent:
                self.send_email(data)
                self._email_sent = True
        return

    def format_data_for_email(self, data):
        """Add the submitted form data to the mail message."""
        items = []
        tpl = u'{0}: {1}\n'
        if 'country' in data:
            items.append(
                tpl.format(
                    translate(_(u'Country'), context=self.request),
                    data['country'],
                ),
            )
        if 'zipcode' in data:
            items.append(
                tpl.format(
                    translate(_(u'ZIP'), context=self.request),
                    data['zipcode'],
                ),
            )
        if 'phone' in data:
            items.append(
                tpl.format(
                    translate(_(u'Phone Number'), context=self.request),
                    data['phone'],
                ),
            )

        if 'arrival_date' in data:
            items.append(
                tpl.format(
                    translate(_(u'Arrival Date'), context=self.request),
                    data['arrival_date'],
                ),
            )
        if 'departure_date' in data:
            items.append(
                tpl.format(
                    translate(_(u'Departure Date'), context=self.request),
                    data['departure_date'],
                ),
            )
        if 'adults' in data:
            items.append(
                tpl.format(
                    translate(_(u'Adults'), context=self.request),
                    data['adults'],
                ),
            )
        if 'children' in data:
            items.append(
                tpl.format(
                    translate(_(u'Children'), context=self.request),
                    data['children'],
                ),
            )
        if 'accept_tcs' in data:
            items.append(
                tpl.format(
                    translate(
                        _(u'Terms & Conditions accepted'),
                        context=self.request,
                    ),
                    data['accept_tcs'],
                ),
            )
        return u''.join(items)

    def send_email(self, data):
        """Construct and send an email message."""
        portal = api.portal.get()

        try:
            email_charset = api.portal.get_registry_record(
                'plone.email_charset',
            )
        except api.exc.InvalidParameterError:
            email_charset = portal.getProperty('email_charset', 'utf-8')

        recipient = None
        try:
            portal_address = api.portal.get_registry_record(
                'plone.email_from_address',
            )
        except api.exc.InvalidParameterError:
            # Before Plone 5.0b2 these were stored in portal_properties
            portal_address = portal.getProperty('email_from_address', '')
        custom_recipient = getattr(self.data, 'recipient', None)

        if custom_recipient is not None:
            recipient = custom_recipient
        else:
            agent = self.listing_info.get('agent')
            try:
                recipient = agent.get('agent_email').get('value')
            except AttributeError:
                recipient = None

        if recipient is None:
            recipient = portal_address

        recipients = [recipient]
        if getattr(self.data, 'bcc', None) is not None:
            bcc = self.data.bcc
            recipients += [formataddr(addr) for addr in getaddresses((bcc, ))]

        sender = formataddr((data['name'], data['sender_from_address']))

        overridden = self.listing_info.get('overridden', False)
        if overridden is True or custom_recipient is not None:
            orig_agent = self.listing_info.get('original_agent')
            agent = translate(
                EMAIL_TEMPLATE_AGENT,
                context=self.request,
            ).format(
                agent=orig_agent.get('name').get('value'),
                agent_email=orig_agent.get('agent_email').get('value'),
                profile=orig_agent.get('profile'),
            )
            data['message'] = '\n'.join([data['message'], agent])
        data['form_data'] = self.format_data_for_email(data)
        data['url'] = self.request.getURL()

        subject = data['subject']

        body = translate(
            EMAIL_TEMPLATE,
            context=self.request,
        ).format(**data)
        email_msg = message_from_string(body.encode(email_charset))
        email_msg['To'] = formataddr((recipient, recipient))

        api.portal.send_email(
            sender=sender,
            recipient=recipients,
            subject=subject,
            body=email_msg,
        )


# Register Captcha validator for the captcha field in the ICaptchaForm
validator.WidgetValidatorDiscriminators(
    CaptchaValidator, field=IEmailForm['captcha'])


class IAgentContactPortlet(IPortletDataProvider):
    """A portlet which sends an email to the agent."""

    heading = schema.TextLine(
        description=_(
            u'Custom title for the portlet. If no title is provided, the '
            u'default title is used.',
        ),
        required=False,
        title=_(u'Portlet Title'),
    )

    description = schema.Text(
        description=_(u'Additional description for the portlet.'),
        required=False,
        title=_('Description'),
    )

    country_visible = schema.Bool(
        required=False,
        title=_(u'Show Country field in email form?'),
    )

    zipcode_visible = schema.Bool(
        required=False,
        title=_(u'Show ZIP field in email form?'),
    )

    phone_required = schema.Bool(
        required=False,
        title=_(u'Make phone field in email form required?'),
    )

    accept_tcs_visible = schema.Bool(
        required=False,
        title=_(u'Show Accept Terms & Conditions field in email form?'),
    )

    if PLONE_5:
        accept_tcs_target = schema.Choice(
            required=False,
            source=CatalogSource(),
            title=_(u'Terms & Conditions page'),
        )
    elif PLONE_4:
        accept_tcs_target = schema.Choice(
            required=False,
            source=SearchableTextSourceBinder({}, default_query='path:'),
            title=_(u'Terms & Conditions page'),
        )

    captcha_visible = schema.Bool(
        default=True,
        required=False,
        title=_(u'Show Captcha field in email form?'),
    )

    mail_sent_msg = schema.Text(
        description=_(
            u'Thank you message that is shown after the mail was sent.',
        ),
        required=False,
        title=_(u'Mail Sent Message'),
    )

    recipient = schema.TextLine(
        constraint=validate_email,
        description=_(
            u'Override the recipient e-mail address. Leave blank to use the '
            u'e-mail address from the agent information.',
        ),
        required=False,
        title=_(u'Override Recipient'),
    )

    bcc = schema.TextLine(
        description=_(
            u'E-mail addresses which receive a blind carbon copy (comma '
            u'separated).',
        ),
        required=False,
        title=_(u'BCC Recipients'),
    )

    reject_links = schema.Bool(
        default=True,
        description=_(
            u'Activate for Spam Protection. Any attempt to use a link inside '
            u'this form will raise a validation error.',
        ),
        required=False,
        title=_(u'Reject Text with Links?'),
    )


@implementer(IAgentContactPortlet)
class Assignment(base.Assignment):
    """Agent Contact Portlet Assignment."""

    heading = FieldProperty(IAgentContactPortlet['heading'])
    description = FieldProperty(IAgentContactPortlet['description'])
    country_visible = FieldProperty(IAgentContactPortlet['country_visible'])
    zipcode_visible = FieldProperty(IAgentContactPortlet['zipcode_visible'])
    phone_required = FieldProperty(IAgentContactPortlet['phone_required'])
    accept_tcs_visible = FieldProperty(
        IAgentContactPortlet['accept_tcs_visible'],
    )
    accept_tcs_target = None
    captcha_visible = FieldProperty(IAgentContactPortlet['captcha_visible'])
    mail_sent_msg = FieldProperty(IAgentContactPortlet['mail_sent_msg'])
    recipient = FieldProperty(IAgentContactPortlet['recipient'])
    bcc = FieldProperty(IAgentContactPortlet['bcc'])
    reject_links = FieldProperty(IAgentContactPortlet['reject_links'])

    title = _(u'MLS: Agent Contact')

    def __init__(
        self,
        heading=None,
        description=None,
        country_visible=None,
        zipcode_visible=None,
        phone_required=None,
        accept_tcs_visible=None,
        accept_tcs_target=None,
        captcha_visible=None,
        mail_sent_msg=None,
        recipient=None,
        bcc=None,
        reject_links=None,
    ):
        self.heading = heading
        self.description = description
        self.country_visible = country_visible
        self.zipcode_visible = zipcode_visible
        self.phone_required = phone_required
        self.accept_tcs_visible = accept_tcs_visible
        self.accept_tcs_target = accept_tcs_target
        self.captcha_visible = captcha_visible
        self.mail_sent_msg = mail_sent_msg
        self.recipient = recipient
        self.bcc = bcc
        self.reject_links = reject_links


class Renderer(base.Renderer):
    """Agent Contact Portlet Renderer."""

    if PLONE_5:
        render = ViewPageTemplateFile('templates/p5_agent_contact.pt')
    elif PLONE_4:
        render = ViewPageTemplateFile('templates/agent_contact.pt')

    @property
    def already_sent(self):
        return self.form.already_sent

    @property
    def available(self):
        return IListingDetails.providedBy(self.view) and \
            getattr(self.view, 'listing_id', None) is not None

    @property
    def description(self):
        return self.data.description

    @property
    def mail_sent_msg(self):
        return self.data.mail_sent_msg or PMF(u'Mail sent.')

    @property
    def title(self):
        return self.data.heading or _(u'Agent Contact')

    @property
    def can_send_email(self):
        portal = api.portal.get()

        ctrlOverview = api.content.get_view(
            context=portal,
            request=portal.REQUEST,
            name='overview-controlpanel',
        )
        if ctrlOverview.mailhost_warning():
            return False
        return True

    def update(self):
        if self.view.info is None:
            return
        listing_info = {
            'listing_id': self.view.info.get('id').get('value'),
            'listing_title': self.view.info.get('title').get('value'),
            'agent': self.view.contact.get('agent'),
            'original_agent': self.view.contact.get('original_agent'),
            'overridden': self.view.contact.get('overridden', False),
        }

        z2.switch_on(self, request_layer=IFormLayer)
        portlet_hash = self.__portlet_metadata__.get('hash')
        self.form = EmailForm(aq_inner(self.context), self.request,
                              portlet_hash, listing_info, self.data)
        if HAS_WRAPPED_FORM:
            alsoProvides(self.form, IWrappedForm)
        self.form.update()


class AddForm(base.AddForm):
    """Add form for the Agent Contact portlet."""
    if PLONE_5:
        schema = IAgentContactPortlet
    elif PLONE_4:
        form_fields = formlib.form.Fields(IAgentContactPortlet)
        form_fields['accept_tcs_target'].custom_widget = UberSelectionWidget
    label = _(u'Add Agent Contact portlet')
    description = MSG_PORTLET_DESCRIPTION

    def create(self, data):
        assignment = Assignment()
        if PLONE_4:
            formlib.form.applyChanges(assignment, self.form_fields, data)
        return assignment


class EditForm(base.EditForm):
    """Edit form for the Agent Contact portlet"""
    if PLONE_5:
        schema = IAgentContactPortlet
    elif PLONE_4:
        form_fields = formlib.form.Fields(IAgentContactPortlet)
        form_fields['accept_tcs_target'].custom_widget = UberSelectionWidget
    label = _(u'Edit Agent Contact portlet')
    description = MSG_PORTLET_DESCRIPTION



# -*- coding: utf-8 -*-
"""MLS API utility methods."""

from Acquisition import aq_inner
from Acquisition import aq_parent
from DateTime import DateTime
from mls.apiclient.client import ListingResource
from mls.apiclient.exceptions import MLSError
from plone import api
from plone.mls.core.api import get_settings
from plone.mls.listing import PRODUCT_NAME
from plone.mls.listing.browser.localconfig import CONFIGURATION_KEY
from plone.mls.listing.interfaces import ILocalAgencyInfo
from plone.mls.listing.interfaces import IMLSAgencyContactInformation
from plone.registry.interfaces import IRegistry
from Products.CMFPlone.interfaces import IPloneSiteRoot
from zope.annotation.interfaces import IAnnotations
from zope.component import getUtility

import copy
import logging
import time


logger = logging.getLogger(PRODUCT_NAME)
# Store the options here (which means in RAM)
OPTIONS_CACHE = {}  # language_category: ({date, language, category, itemlist})

CONFIGURATION_KEYS = [
    'plone.mls.listing.recentlistings',
    'plone.mls.listing.listingsearch',
    'plone.mls.listing.listingcollection',
]


def _remove_omitted(params, omit):
    """Removed omitted keys from the params dict."""
    if omit is not None and isinstance(omit, (list, tuple)):
        for item in omit:
            try:
                del params[item]
            except KeyError:
                continue


def prepare_search_params(data, omit=None):
    """Prepare search params."""
    params = {}

    _remove_omitted(data, omit)

    for item in data:
        if item == 'interior_area':
            # change data and key to 'floor_area' instead of 'interior_area'
            item = 'floor_area'
            data[item] = data.get('interior_area')
            data.pop('interior_area', None)

        if item in ['baths', 'beds', 'lot_size', 'floor_area']:
            min_max = data[item]
            if isinstance(min_max, (list, tuple)):
                if len(min_max) > 0 and min_max[0] != '--MINVALUE--':
                    params[item + '_min'] = min_max[0]
                if len(min_max) > 1 and min_max[1] != '--MAXVALUE--':
                    params[item + '_max'] = min_max[1]
                continue

        # Convert lists and tuples to comma separated lists.
        if isinstance(data[item], (list, tuple)):
            if len(data.get(item, ())) > 0:
                data[item] = ','.join(data[item])
            else:
                data[item] = None

        # Remove all None-Type values.
        if data[item] is not None:
            value = data[item]
            if isinstance(value, unicode):
                value = value.encode('utf-8')
            params[item] = value
    return params


class SearchOptions(object):
    """Cached search options."""

    # time in minutes after which we retry to load it after a failure
    FAILURE_DELAY = 10
    category = None
    language = None
    timeout = 100

    def __init__(self, category, language, timeout, context=None):
        self.category = category
        self.language = language
        self.timeout = timeout
        self.context = context

        self._items = []
        self._loaded = False  # Is the category already loaded?
        self._failed = False  # Does it fail with the last update?
        self._last_update_time_in_minutes = 0  # When was the last update?
        self._last_update_time = None  # Time as DateTime or Now.

    @property
    def last_update_time_in_minutes(self):
        """return the time the last update was done in minutes"""
        return self._last_update_time_in_minutes

    @property
    def last_update_time(self):
        """return the time the last update was done in minutes"""
        return self._last_update_time

    @property
    def update_failed(self):
        return self._failed

    @property
    def ok(self):
        return (not self._failed and self._loaded)

    @property
    def loaded(self):
        """return whether this feed is loaded or not"""
        return self._loaded

    @property
    def needs_update(self):
        """check if this feed needs updating"""
        now = time.time() / 60
        return (self.last_update_time_in_minutes + self.timeout) < now

    def update(self):
        """update this feed"""
        now = time.time() / 60

        # check for failure and retry
        if self.update_failed:
            if (self.last_update_time_in_minutes + self.FAILURE_DELAY) < now:
                return self._retrieveCategory()
            else:
                return False

        # check for regular update
        if self.needs_update:
            return self._retrieveCategory()

        return self.ok

    def _retrieveCategory(self):
        """do the actual work and try to retrieve the feed"""
        if self.category is not None:
            self._last_update_time_in_minutes = time.time() / 60
            self._last_update_time = DateTime()
            settings = get_settings(context=self.context)
            base_url = settings.get('mls_site', None)
            api_key = settings.get('mls_key', None)
            debug = api.env.debug_mode
            resource = ListingResource(base_url, api_key=api_key, debug=debug)
            results = []
            try:
                results = resource.category(self.category, self.language)
            except MLSError as e:
                self._loaded = True  # we tried at least but have a failed load
                self._failed = True
                logger.warn(e)
                return False
            self._items = results
            self._loaded = True
            self._failed = False
            return True
        self._loaded = True
        self._failed = True  # no url set means failed
        return False  # no url set, although that should not really happen

    @property
    def items(self):
        return self._items


def search_options(mls_url, category, lang=None, context=None):
    if mls_url is None or len(mls_url) < 1:
        return

    timeout = 60
    key = mls_url + category + '_' + str(lang)
    options = OPTIONS_CACHE.get(key, None)

    if options is None:
        options = SearchOptions(category, lang, timeout, context=context)

        if options.update():
            OPTIONS_CACHE[key] = options
    else:
        options.update()

    return options.items


def recent_listings(params={}, batching=True, context=None, config=None):
    """Return a list of recent MLS listings."""
    search_params = {
        'sort_on': 'last_activated_date',
        'reverse': '1',
    }
    search_params.update(params)
    return search(
        params=search_params,
        batching=batching,
        context=context,
        config=config,
    )


def get_configs(context=None, merged=False):
    """Return all available configurations."""
    result = {}
    if not context:
        return result
    annotations = IAnnotations(context)
    for key in CONFIGURATION_KEYS:
        config = annotations.get(key, {})
        if merged:
            result.update(config)
        else:
            result[key] = config
    return result


def listing_details(listing_id, lang=None, context=None):
    """Return detail information for a listing."""
    settings = get_settings(context=context)
    base_url = settings.get('mls_site', None)
    api_key = settings.get('mls_key', None)
    debug = api.env.debug_mode
    config = get_configs(context=context, merged=True)
    params = {}
    if config.get('show_unverified', False):
        params['apiowner'] = settings.get('agency_id')
        params['show_unverified'] = True
    if config.get('show_unverified_only', False):
        params['show_unverified_only'] = True
    resource = ListingResource(base_url, api_key=api_key, debug=debug)
    try:
        listing = resource.get(listing_id, lang=lang, params=params)
    except MLSError as e:
        logger.warn(e)
        return None
    listing = listing.get('listing', None)
    if listing is not None:
        agent = copy.deepcopy(listing.get('contact', {}).get('agent'))
        listing['original_agent'] = agent
    return listing


def search(params={}, batching=True, context=None, config=None):
    """Search for listings."""
    if config is None:
        config = {}
    settings = get_settings(context=context)
    search_params = {
        'sort_on': 'last_activated_date',
        'reverse': '1',
    }
    if config.get('show_unverified', False):
        search_params['apiowner'] = settings.get('agency_id')
    listing_types = set(config.get('listing_type', ()))
    if listing_types:
        # Available Listing Types are restricted.
        search_listing_types = params.get('listing_type', None)
        if search_listing_types:
            search_listing_types = set(search_listing_types.split(','))
            listing_types = listing_types.intersection(search_listing_types)
        params['listing_type'] = ','.join(listing_types)
    agency_listings = params.pop('agency_listings', False)
    agency_priority = params.get('agency_priority', False)
    if agency_listings is True or agency_priority is True:
        agency_id = params.pop('overriding_agency_id', None)
        if not agency_id:
            agency_id = settings.get('agency_id', None)
        agency_id = agency_id.replace(' ', '')
        search_params['agency_id'] = agency_id
    search_params.update(params)
    base_url = settings.get('mls_site', None)
    api_key = settings.get('mls_key', None)
    batch = None
    results = []
    debug = api.env.debug_mode
    resource = ListingResource(base_url, api_key=api_key, debug=debug)

    try:
        results, batch = resource.search(search_params)
    except MLSError as e:
        logger.warn(e)

    if batching:
        return results, batch
    return results


def _local_agency_info(context):
    """Get local agency information."""
    settings = None
    obj = context
    while (
            not IPloneSiteRoot.providedBy(obj) and
            not ILocalAgencyInfo.providedBy(obj)):
        parent = aq_parent(aq_inner(obj))
        if parent is None:
            return
        obj = parent
    if ILocalAgencyInfo.providedBy(obj):
        annotations = IAnnotations(obj)
        settings = annotations.get(
            CONFIGURATION_KEY, annotations.setdefault(CONFIGURATION_KEY, {}))
    return settings


def get_agency_info(context=None):
    """Get the agency information."""
    if context is None:
        context = api.portal.get()

    local_info = _local_agency_info(context)
    if local_info is not None:
        logger.debug('Returning local agency information.')
        return local_info

    # Get the global agency info.
    settings = {}
    registry = getUtility(IRegistry)
    if registry is not None:
        try:
            registry_settings = registry.forInterface(  # noqa
                IMLSAgencyContactInformation,
            )
        except Exception:
            logger.warning('Global agency information not available.')
        else:
            settings = dict([
                (a, getattr(registry_settings, a)) for a in
                registry_settings.__schema__
            ])
            logger.debug('Returning global agency information.')
    if not settings.get('use_custom_info', False):
        return
    return settings



# -*- coding: utf-8 -*-
"""Agent Information Portlet."""

from plone.app.portlets.portlets import base
from plone.mls.listing import PLONE_4
from plone.mls.listing import PLONE_5
from plone.mls.listing.browser.interfaces import IListingDetails
from plone.mls.listing.i18n import _
from plone.mls.listing.portlets.agent_contact import IAgentContactPortlet
from plone.portlets.interfaces import IPortletDataProvider
from plone.portlets.interfaces import IPortletManager
from plone.portlets.interfaces import IPortletRetriever
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope import schema
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty


if PLONE_4:
    from zope.formlib import form


MSG_PORTLET_DESCRIPTION = _(
    u'This portlet shows the corresponding agent information for a given '
    u'listing. Note that this portlet is only available for the detail view '
    u'of a listing.',
)


class IAgentInformationPortlet(IPortletDataProvider):
    """A portlet displaying agent information for a listing."""

    heading = schema.TextLine(
        description=_(
            u'Custom title for the portlet. If no title is provided, the '
            u'default title is used.'),
        required=False,
        title=_(u'Portlet Title'),
    )


@implementer(IAgentInformationPortlet)
class Assignment(base.Assignment):
    """Agent Information Portlet Assignment."""

    heading = FieldProperty(IAgentInformationPortlet['heading'])
    title = _(u'MLS: Agent Information')

    def __init__(self, heading=None):
        self.heading = heading


class Renderer(base.Renderer):
    """Agent Information Portlet Renderer."""

    if PLONE_5:
        render = ViewPageTemplateFile('templates/p5_agent_information.pt')
    elif PLONE_4:
        render = ViewPageTemplateFile('templates/agent_information.pt')

    @property
    def available(self):
        return IListingDetails.providedBy(self.view) and \
            getattr(self.view, 'listing_id', None) is not None

    @property
    def title(self):
        return self.data.heading or _(u'Agent Information')

    @property
    def agent_contact_portlet_available(self):
        for column in ['plone.leftcolumn', 'plone.rightcolumn']:
            manager = getUtility(IPortletManager, name=column)
            retriever = getMultiAdapter(
                (self.context, manager),
                IPortletRetriever,
            )
            portlets = retriever.getPortlets()
            for portlet in portlets:
                if IAgentContactPortlet.providedBy(portlet['assignment']):
                    return True
        return False


class AddForm(base.AddForm):
    """Add form for the Agent Information portlet."""
    if PLONE_5:
        schema = IAgentInformationPortlet
    elif PLONE_4:
        form_fields = form.Fields(IAgentInformationPortlet)
    label = _(u'Add Agent Information portlet')
    description = MSG_PORTLET_DESCRIPTION

    def create(self, data):
        assignment = Assignment()
        if PLONE_4:
            form.applyChanges(assignment, self.form_fields, data)
        return assignment


class EditForm(base.EditForm):
    """Edit form for the Agent Information portlet."""
    if PLONE_5:
        schema = IAgentInformationPortlet
    elif PLONE_4:
        form_fields = form.Fields(IAgentInformationPortlet)
    label = _(u'Edit Agent Information portlet')
    description = MSG_PORTLET_DESCRIPTION



# -*- coding: utf-8 -*-
"""Listing Quick Search Portlet."""

from Acquisition import aq_inner
from plone import api
from plone.app.portlets.portlets import base
from plone.directives import form
from plone.mls.listing import PLONE_4
from plone.mls.listing import PLONE_5
from plone.mls.listing.browser import listing_search
from plone.mls.listing.i18n import _
from plone.portlets.interfaces import IPortletDataProvider
from plone.z3cform import z2
from Products.CMFPlone import PloneMessageFactory as PMF
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from z3c.form import button
from z3c.form.interfaces import IFormLayer
from zope import schema
from zope.annotation.interfaces import IAnnotations
from zope.interface import alsoProvides
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty


# starting from 0.6.0 version plone.z3cform has IWrappedForm interface
try:
    from plone.z3cform.interfaces import IWrappedForm
    HAS_WRAPPED_FORM = True
except ImportError:
    HAS_WRAPPED_FORM = False

if PLONE_4:
    from zope import formlib

MSG_PORTLET_DESCRIPTION = _(u'This portlet shows a listing quick search form.')

#: Definition of available fields in the given ``rows``.
FIELD_ORDER = {
    'row_listing_type': [
        'q',
        'listing_type',
    ],
    'row_location': [
        'location_state',
        'location_county',
        'location_district',
        'location_city',
    ],
    'row_beds_baths': [
        'beds',
        'baths',
    ],
    'row_object_type': [
        'object_type',
    ],
    'row_price': [
        'price_min',
        'price_max',
    ],
    'row_sizes': [
        'lot_size',
        'interior_area',
    ],
    'row_filter': [
        'air_condition',
        'pool',
        'jacuzzi',
        'location_type',
        'geographic_type',
    ],
}


class QuickSearchForm(form.SchemaForm):
    """Quick Search Form."""

    if PLONE_5:
        template = ViewPageTemplateFile('templates/p5_search_form.pt')
    elif PLONE_4:
        template = ViewPageTemplateFile('templates/search_form.pt')

    schema = listing_search.IListingSearchForm
    ignoreContext = True
    method = 'get'

    def __init__(self, context, request, data=None):
        """Customized form constructor.

        This one also takes an optional ``data`` attribute so it can be
        instantiated from within a portlet without loosing access to the
        portlet data.
        """
        super(QuickSearchForm, self).__init__(context, request)
        self.data = data
        form_context = self.getContent()
        if form_context is not None:
            self.prefix = 'form.{0}'.format(form_context.id)
        self.omitted = []

    @property
    def config(self):
        """Get view configuration data from annotations."""
        annotations = IAnnotations(self.getContent())
        return annotations.get(listing_search.CONFIGURATION_KEY, {})

    def getContent(self):
        search_path = self.data.target_search
        if search_path is None:
            return self.context

        if search_path.startswith('/'):
            obj = api.content.get(path=search_path)
        else:
            obj = api.content.get(UID=search_path)

        return obj

    def update(self):
        super(QuickSearchForm, self).update()
        if self.config.get('location_city', None):
            self.omitted.extend([
                'location_state',
                'location_county',
                'location_district',
                'location_city',
            ])

        if self.config.get('location_district', None):
            self.omitted.extend([
                'location_state',
                'location_county',
                'location_district',
            ])
        elif self.config.get('location_county', None):
            self.omitted.extend([
                'location_state',
                'location_county',
                'location_district',
            ])
        elif self.config.get('location_state', None):
            self.omitted.extend([
                'location_state',
                'location_county',
                'location_district',
            ])

        # Hide those 2 fields from quick search.
        if not self.show_filter:
            self.omitted.extend([
                'location_county',
                'location_district',
            ])

    @button.buttonAndHandler(PMF(u'label_search', default=u'Search'),
                             name='search')
    def handle_search(self, action):
        """Search button."""
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

    @property
    def action(self):
        """See interfaces.IInputForm."""
        search_path = self.data.target_search

        if search_path is None:
            return

        if search_path.startswith('/'):
            obj = api.content.get(path=search_path)
        else:
            obj = api.content.get(UID=search_path)

        if obj:
            return obj.absolute_url()

    def _widgets(self, row):
        """Return a list of widgets that should be shown for a given row."""
        widget_data = dict(self.widgets.items())
        available_fields = FIELD_ORDER.get(row, [])
        return [
            widget_data.get(key, None) for key in available_fields if
            key not in self.omitted
        ]

    @property
    def show_filter(self):
        """Decide if the filter should be shown or not."""
        form = self.request.form
        button = '{0}.buttons.search'.format(self.prefix)
        return (
            listing_search.IListingSearch.providedBy(self.context) and
            (button in form.keys() or self.config.get('auto_search', False))
        )

    def widgets_listing_type(self):
        """Return the widgets for the row ``row_listing_type``."""
        return self._widgets('row_listing_type')

    def widgets_location(self):
        """Return the widgets for the row ``row_location``."""
        return self._widgets('row_location')

    def widgets_beds_baths(self):
        """Return the widgets for the row ``row_beds_baths``."""
        return self._widgets('row_beds_baths')

    def widgets_object_type(self):
        """Return the widgets for the row ``row_object_type``."""
        return self._widgets('row_object_type')

    def widgets_price(self):
        """Return the widgets for the row ``row_price``."""
        return self._widgets('row_price')

    def widgets_sizes(self):
        """Return the widgets for the row ``row_sizes``."""
        return self._widgets('row_sizes')

    def widgets_filter(self):
        """Return the widgets for the row ``row_filter``."""
        return self._widgets('row_filter')

    def widgets_filter_other(self):
        """Return all other widgets that have not been shown until now."""
        defined_fields = FIELD_ORDER.values()
        shown_fields = [
            shown_field for field_lists in defined_fields for
            shown_field in field_lists
        ]
        return [
            widget for field_name, widget in self.widgets.items() if
            field_name not in shown_fields
        ]


class IQuickSearchPortlet(IPortletDataProvider):
    """A portlet displaying a listing quick search form."""

    heading = schema.TextLine(
        description=_(
            u'Custom title for the portlet (search mode). If no title is '
            u'provided, the default title is used.',
        ),
        required=False,
        title=_(u'Portlet Title (Search)'),
    )

    heading_filter = schema.TextLine(
        description=_(
            u'Custom title for the portlet (filter mode). If no title is '
            u'provided, the default title is used.',
        ),
        required=False,
        title=_(u'Portlet Title (Filter)'),
    )

    target_search = schema.Choice(
        description=_(
            u'Find the search page which will be used to show the results.',
        ),
        required=True,
        vocabulary='plone.mls.listing.available_searches',
        title=_(u'Search Page'),
    )


@implementer(IQuickSearchPortlet)
class Assignment(base.Assignment):
    """Quick Search Portlet Assignment."""

    heading = FieldProperty(IQuickSearchPortlet['heading'])
    heading_filter = FieldProperty(IQuickSearchPortlet['heading_filter'])
    target_search = None

    title = _(u'MLS: Listing Quick Search')
    mode = 'SEARCH'

    def __init__(self, heading=None, heading_filter=None, target_search=None):
        self.heading = heading
        self.heading_filter = heading_filter
        self.target_search = target_search


class Renderer(base.Renderer):
    """Listing Quick Search Portlet Renderer."""

    if PLONE_5:
        render = ViewPageTemplateFile('templates/p5_quick_search.pt')
    elif PLONE_4:
        render = ViewPageTemplateFile('templates/quick_search.pt')

    @property
    def available(self):
        """Check the portlet availability."""
        search_view = self._search_context()
        return (
            listing_search.IListingSearch.providedBy(search_view) and
            self.mode != 'HIDDEN'
            # self.view.context != search_view
        )

    @property
    def config(self):
        """Get view configuration data from annotations."""
        annotations = IAnnotations(self._search_context())
        return annotations.get(listing_search.CONFIGURATION_KEY, {})

    def _search_context(self):
        search_path = self.data.target_search

        if search_path is None:
            return

        if search_path.startswith('/'):
            obj = api.content.get(path=search_path)
        else:
            obj = api.content.get(UID=search_path)

        return obj

    @property
    def title(self):
        """Return the title dependend on the mode that we are in."""
        if self.mode == 'SEARCH':
            return self.data.heading or _(u'Listing Search')
            return self.data.title
        if self.mode == 'FILTER':
            return self.data.heading_filter or _(u'Filter Results')

    @property
    def mode(self):
        """Return the mode that we are in.

        This can be either ``FILTER`` if a search was already performed and we
        are on a search page or ``SEARCH`` otherwise.
        """
        search_context = self._search_context()
        if search_context != self.context:
            return 'SEARCH'

        if search_context:
            prefix = 'form.{0}'.format(search_context.id)
        else:
            prefix = 'form'
        button = '{0}.buttons.search'.format(prefix)
        form = self.request.form
        if (
            listing_search.IListingSearch.providedBy(self.context) and
            (button in form.keys() or self.config.get('auto_search', False))
        ):
            return 'FILTER'
        elif (
            listing_search.IListingSearch.providedBy(self.context) and
            button not in form.keys()
        ):
            return 'HIDDEN'
        else:
            return 'SEARCH'

    def update(self):
        z2.switch_on(self, request_layer=IFormLayer)
        self.form = QuickSearchForm(aq_inner(self.context), self.request,
                                    self.data)
        if HAS_WRAPPED_FORM:
            alsoProvides(self.form, IWrappedForm)
        self.form.update()


class AddForm(base.AddForm):
    """Add form for the Listing Quick Search portlet."""

    if PLONE_5:
        schema = IQuickSearchPortlet
    elif PLONE_4:
        form_fields = formlib.form.Fields(IQuickSearchPortlet)

    label = _(u'Add Listing Quick Search portlet')
    description = MSG_PORTLET_DESCRIPTION

    def create(self, data):
        assignment = Assignment()
        if PLONE_4:
            formlib.form.applyChanges(assignment, self.form_fields, data)
        return assignment


class EditForm(base.EditForm):
    """Edit form for the Listing Quick Search portlet."""

    if PLONE_5:
        schema = IQuickSearchPortlet
    elif PLONE_4:
        form_fields = formlib.form.Fields(IQuickSearchPortlet)

    label = _(u'Edit Listing Quick Search portlet')
    description = MSG_PORTLET_DESCRIPTION

    def update(self):
        if self.context.target_search.startswith('/'):
            obj = api.content.get(path=self.context.target_search)
            if obj is not None:
                uid = api.content.get_uuid(obj)
                self.context.target_search = uid
        super(EditForm, self).update()



# -*- coding: utf-8 -*-
"""ValueRange Widget Implementation"""

from plone.mls.listing.browser.valuerange.interfaces import IValueRangeWidget
from plone.mls.listing.i18n import _
from z3c.form.browser.widget import addFieldClass
from z3c.form.browser.widget import HTMLTextInputWidget
from z3c.form.interfaces import IFieldWidget
from z3c.form.interfaces import IFormLayer
from z3c.form.interfaces import NO_VALUE
from z3c.form.widget import FieldWidget
from z3c.form.widget import SequenceWidget
from zope.component import adapter
from zope.i18n import translate
from zope.interface import implementer
from zope.interface import implementsOnly
from zope.schema.interfaces import IField
from zope.schema.interfaces import ITitledTokenizedTerm


class ValueRangeWidget(HTMLTextInputWidget, SequenceWidget):
    """Value range widget."""
    implementsOnly(IValueRangeWidget)

    klass = u'value-range-widget pat-select2'
    prompt = False

    noValueMessage = _('All')
    promptMessage = _('Select a value...')

    # Internal attributes
    _adapterValueAttributes = SequenceWidget._adapterValueAttributes + \
        ('noValueMessage', 'promptMessage', 'prompt')

    def update(self):
        super(ValueRangeWidget, self).update()
        addFieldClass(self)

    @property
    def maximums(self):
        if self.terms is None:  # update() has not been called yet
            return []
        items = []
        selected = self.maximum

        for count, term in enumerate(self.terms):
            if term.token == '--MINVALUE--':
                continue
            id = '{0}-{1:d}'.format(self.id, count)
            content = term.title or term.value
            if ITitledTokenizedTerm.providedBy(term):
                content = translate(
                    term.title, context=self.request, default=term.title)
            items.append({
                'id': id,
                'value': term.token,
                'content': content,
                'selected': term.token == selected})

#         items.append({
#             'id': self.id + '-novalue',
#             'value': '--MAXVALUE--',
#             'content': u"Max",
#             'selected': selected is None or selected == '--MAXVALUE--',
#         })

        return items

    @property
    def minimums(self):
        if self.terms is None:  # update() has not been called yet
            return ()
        items = []
        selected = self.minimum

#         items.append({
#             'id': self.id + '-novalue',
#             'value': '--MINVALUE--',
#             'content': u"Min",
#             'selected': selected is None or selected == 0,
#         })

        for count, term in enumerate(self.terms):
            if term.token == '--MAXVALUE--':
                continue
            id = '{0}-{1:d}'.format(self.id, count)
            content = term.title or term.value
            if ITitledTokenizedTerm.providedBy(term):
                content = translate(
                    term.title, context=self.request, default=term.title)
            items.append({
                'id': id,
                'value': term.token,
                'content': content,
                'selected': term.token == selected})
        return items

    @property
    def maximum(self):
        maximum = self.request.get(self.name + '-max', None)
        if maximum:
            return maximum
        else:
            return u'--MAXVALUE--'

    @property
    def minimum(self):
        minimum = self.request.get(self.name + '-min', None)
        if minimum:
            return minimum
        else:
            return u'--MINVALUE--'

    def extract(self, default=NO_VALUE):
        """See z3c.form.interfaces.IWidget."""
        min_ = self.request.get(self.name + '-min', default)
        max_ = self.request.get(self.name + '-max', default)

        if isinstance(min_, list):
            min_ = min_[0]
        if isinstance(max_, list):
            max_ = max_[0]

        try:
            self.terms.getTermByToken(min_)
        except LookupError:
            min_ = default

        try:
            self.terms.getTermByToken(max_)
        except LookupError:
            max_ = default

        if min_ == default and max_ == default:
            return default

        if min_ is None and max_ is None:
            return default

        return (min_, max_)


@adapter(IField, IFormLayer)
@implementer(IFieldWidget)
def ValueRangeFieldWidget(field, request):
    """IFieldWidget factory for ValueRangeWidget."""
    return FieldWidget(field, ValueRangeWidget(request))



# -*- coding: utf-8 -*-
"""Terms & Conditions Widget Implementation"""

from plone import api
from plone.mls.listing.browser.tcwidget.interfaces import ITCWidget
from Products.Five.browser import BrowserView
from Products.Five.browser.metaconfigure import ViewMixinForTemplates
from z3c.form.browser.checkbox import SingleCheckBoxWidget
from z3c.form.interfaces import IFieldWidget
from z3c.form.interfaces import IFormLayer
from z3c.form.widget import FieldWidget
from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile
from zope.component import adapter
from zope.interface import implementer
from zope.interface import implementer_only
from zope.schema.interfaces import IBool


class RenderTCWidget(ViewMixinForTemplates, BrowserView):
    index = ViewPageTemplateFile('tcwidgetwrapper.pt')


@implementer_only(ITCWidget)
class TCWidget(SingleCheckBoxWidget):
    """Single Input type checkbox widget implementation."""

    klass = u'terms-conditions-widget'
    target = None

    def tc_link(self):
        if not self.target:
            return

        path = str(self.target)
        if path.startswith('/'):
            item = api.content.get(path=path)
        else:
            item = api.content.get(UID=path)
        if not item:
            return
        return {
            'label': item.title,
            'url': item.absolute_url(),
        }


@adapter(IBool, IFormLayer)
@implementer(IFieldWidget)
def TCFieldWidget(field, request):
    """IFieldWidget factory for TCWidget."""
    widget = FieldWidget(field, TCWidget(request))
    # widget.label = u''  # don't show the label twice
    return widget



# -*- coding: utf-8 -*-
"""Terms & Conditions Widget Interface."""

# zope imports
from z3c.form.interfaces import ISingleCheckBoxWidget


class ITCWidget(ISingleCheckBoxWidget):
    """Custom Terms & Conditions widget."""



# -*- coding: utf-8 -*-
"""Local agency information settings."""

# zope imports
from plone.directives import form
from plone.mls.listing.i18n import _
from plone.mls.listing.interfaces import ILocalAgencyInfo
from plone.mls.listing.interfaces import IMLSAgencyContactInformation
from plone.mls.listing.interfaces import IPossibleLocalAgencyInfo
from z3c.form import button
from zope.annotation.interfaces import IAnnotations
from zope.interface import alsoProvides
from zope.interface import noLongerProvides
from zope.traversing.browser.absoluteurl import absoluteURL


CONFIGURATION_KEY = 'plone.mls.listing.localagencyinfo'


class LocalAgencyInfo(form.SchemaForm):
    """Local agency information form."""

    schema = IMLSAgencyContactInformation
    label = _(u'Local Agency Information')
    description = _(
        u'This agency information will be used for this content item and all '
        u'possible child elements.',
    )

    def getContent(self):
        """Get the annotations with the local agency information."""
        annotations = IAnnotations(self.context)
        return annotations.get(
            CONFIGURATION_KEY,
            annotations.setdefault(CONFIGURATION_KEY, {}),
        )

    @button.buttonAndHandler(_(u'Save'))
    def handle_save(self, action):
        """Save the new configuration to the context annotations."""
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        annotations = IAnnotations(self.context)
        annotations[CONFIGURATION_KEY] = data
        self.request.response.redirect(absoluteURL(self.context, self.request))

    @button.buttonAndHandler(_(u'Cancel'))
    def handle_cancel(self, action):
        """Discard changes."""
        self.request.response.redirect(absoluteURL(self.context, self.request))


class LocalAgencyInfoStatus(object):
    """Return activation/deactivation status of the local agency infos."""

    def __init__(self, context, request):
        self.context = context
        self.request = request

    @property
    def can_activate(self):
        """Can the local agency information be activated for this context?"""
        return IPossibleLocalAgencyInfo.providedBy(self.context) and \
            not ILocalAgencyInfo.providedBy(self.context)

    @property
    def active(self):
        """Is the local agency information active for this context?"""
        return ILocalAgencyInfo.providedBy(self.context)


class LocalAgencyInfoToggle(object):
    """Toggle local agency information for the current context."""

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def __call__(self):
        """Perform the toggle."""
        msg_type = 'info'

        if ILocalAgencyInfo.providedBy(self.context):
            # Deactivate local agency information.
            noLongerProvides(self.context, ILocalAgencyInfo)
            self.context.reindexObject(idxs=['object_provides'])
            msg = _(u'Local agency information deactivated.')
        elif IPossibleLocalAgencyInfo.providedBy(self.context):
            alsoProvides(self.context, ILocalAgencyInfo)
            self.context.reindexObject(idxs=['object_provides'])
            msg = _(u'Local agency information activated.')
        else:
            msg = _(
                u'The local agency information don\'t work with this '
                u'content type. Add \'IPossibleLocalAgencyInfo\' to the '
                u'provided interfaces to enable this feature.',
            )
            msg_type = 'error'

        self.context.plone_utils.addPortalMessage(msg, msg_type)
        self.request.response.redirect(self.context.absolute_url())
